'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var rollup$1 = _interopDefault(require('rollup'));

/** @typedef { import("rollup").RollupOptions } RollupOptions */

const { rollup } = rollup$1;

/**
 * Bundle package at 'id'
 *
 * @param { string } inputPath
 * @param { string } outputPath
 * @param { string } sourcePrefix
 * @param { RollupOptions } rollupOptions
 * @param { (err?: Error) => void } fn
 * @returns { Promise<void> }
 */
var bundleWorker = async function bundle(
  inputPath,
  outputPath,
  sourcePrefix,
  rollupOptions,
  fn,
) {
  const parsedOptions = parseOptions(sourcePrefix + inputPath, rollupOptions);
  const inputOptions = {
    ...parsedOptions.input,
    input: inputPath,
  };
  const outputOptions = {
    ...parsedOptions.output,
    file: outputPath,
  };

  try {
    const bundled = await rollup(inputOptions);
    await bundled.write(outputOptions);
    fn();
  } catch (err) {
    console.log(err);
    fn(err);
  }
};

/**
 * Parse Rollup options
 *
 * @param { string } banner
 * @param { RollupOptions } options
 * @returns { { input: object, output: object } }
 */
function parseOptions(banner, options) {
  if (!options) {
    return { input: {}, output: { banner } };
  }

  const { input, treeshake, output = {}, watch, ...inputOverride } = options;
  // @ts-ignore
  const { file, format, sourcemap, ...outputOverride } = output;

  if ('banner' in outputOverride) {
    outputOverride.banner = banner + outputOverride.banner;
  } else {
    // @ts-ignore
    outputOverride.banner = banner;
  }

  return { input: inputOverride, output: outputOverride };
}

module.exports = bundleWorker;
